/*
 * Copyright (C) 2019-2022, Xilinx, Inc.
 * Copyright (C) 2022-2024, Advanced Micro Devices, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
#ifndef _DSPLIB_FUNC_APPROX_REF_HPP_
#define _DSPLIB_FUNC_APPROX_REF_HPP_

/*
Function Approximation reference model
*/

#include <adf.h>
#include <limits>
#include "fir_ref_utils.hpp"

using namespace adf;

#define _DSPLIB_FUNC_APPROX_REF_DEBUG_

#define SQRT_FUNC 0
#define INVSQRT_FUNC 1
#define LOG_FUNC 2
#define EXP_FUNC 3
#define INV_FUNC 4

namespace xf {
namespace dsp {
namespace aie {
namespace func_approx {

//-----------------------------------------------------------------------------------------------------
template <typename TT_DATA,
          unsigned int TP_COARSE_BITS,
          unsigned int TP_FINE_BITS,
          unsigned int TP_DOMAIN_MODE,
          unsigned int TP_WINDOW_VSIZE,
          unsigned int TP_SHIFT,
          unsigned int TP_RND,
          unsigned int TP_SAT,
          unsigned int TP_FUNC_CHOICE>
class func_approx_ref {
   private:
   public:
    // Constructor
    func_approx_ref() {}

    // Register Kernel Class
    static void registerKernelClass() { REGISTER_FUNCTION(func_approx_ref::approx_main); }

    void approx_main(input_buffer<TT_DATA>& inWindow, output_buffer<TT_DATA>& outWindow);
};
}
}
}
}

#endif // _DSPLIB_FUNC_APPROX_REF_HPP_
