//<copyright-disclaimer-start>
//  **************************************************************************************************************
//  * © 2025 Advanced Micro Devices, Inc. All rights reserved.                                                   *
//  * DISCLAIMER                                                                                                 *
//  * The information contained herein is for informational purposes only, and is subject to change              *
//  * without notice. While every precaution has been taken in the preparation of this document, it              *
//  * may contain technical inaccuracies, omissions and typographical errors, and AMD is under no                *
//  * obligation to update or otherwise correct this information.  Advanced Micro Devices, Inc. makes            *
//  * no representations or warranties with respect to the accuracy or completeness of the contents of           *
//  * this document, and assumes no liability of any kind, including the implied warranties of noninfringement,  *
//  * merchantability or fitness for particular purposes, with respect to the operation or use of AMD            *
//  * hardware, software or other products described herein.  No license, including implied or                   *
//  * arising by estoppel, to any intellectual property rights is granted by this document.  Terms and           *
//  * limitations applicable to the purchase or use of AMD’s products are as set forth in a signed agreement     *
//  * between the parties or in AMD's Standard Terms and Conditions of Sale. GD-18                               *
//  *                                                                                                            *
//  **************************************************************************************************************
//<copyright-disclaimer-end>


#pragma once

#include "system_settings.h"

using namespace adf;

namespace filtering = xf::dsp::aie::fir;

class FirGraph: public graph
{
private:
	// Channel Filter coefficients
	std::vector<int16> chan_taps = std::vector<int16>{
		   -3,     3,    -3,     3,    -3,     4,    -4,     4,    -4,     4,
			 -4,     5,    -5,     5,    -5,     6,    -6,     6,    -6,     7,
			 -7,     8,    -8,     9,    -9,    10,   -11,    11,   -12,    12,
			-14,    15,   -16,    17,   -18,    20,   -23,    25,   -25,    27,
	    -32,    38,   -41,    43,   -47,    58,   -72,    81,   -85,    97,
		 -132,   185,  -227,   245,  -303,   555, -1170,  2181, -3383,  4376,
		28004};

	// HalfBand Filter coefficients
	std::vector<int16> hb1_taps = std::vector<int16>{
	      435, -3345, 19294, 32767};

	std::vector<int16> hb2_taps = std::vector<int16>{
			-162,  1112, -4436, 19870, 32767};

	std::vector<int16> hb3_taps = std::vector<int16>{
      7,   -22,    55,  -114,   213,  -369,   606,  -956,  1475, -2274,  3638, -6620, 20745, 32767};


	//FIR Graph class
	filtering::decimate_hb::fir_decimate_hb_graph<DATA_TYPE, COEFF_TYPE, FIR_LEN_HB1, SHIFT_HB1,	ROUND_MODE_HB1, WINDOW_SIZE1> hb1;
	filtering::decimate_hb::fir_decimate_hb_graph<DATA_TYPE, COEFF_TYPE, FIR_LEN_HB2, SHIFT_HB2,	ROUND_MODE_HB2, WINDOW_SIZE2> hb2;
	filtering::decimate_hb::fir_decimate_hb_graph<DATA_TYPE, COEFF_TYPE, FIR_LEN_HB3, SHIFT_HB3,	ROUND_MODE_HB3, WINDOW_SIZE3> hb3;

	filtering::sr_sym::fir_sr_sym_graph<DATA_TYPE, COEFF_TYPE, FIR_LEN_CHAN, SHIFT_CHAN,	ROUND_MODE_CHAN, WINDOW_SIZE_CHAN> chan;

public:
	input_plio in;
	output_plio out;

	// Constructor - with FIR graph class initialization
	FirGraph():hb1(hb1_taps),hb2(hb2_taps),hb3(hb3_taps),chan(chan_taps) {
	     	in  = input_plio::create("plDataIn1", plio_32_bits,"data/input32.txt");
      		out = output_plio::create("plDataOut1", plio_32_bits,"data/output32.txt");	// Make connections
		// Margin gets automatically added within the FIR graph class.
		// Margin equals to FIR length rounded up to nearest multiple of 32 Bytes.

		connect<>(in.out[0], hb1.in[0]);
		connect<>(hb1.out[0], hb2.in[0]);
		connect<>(hb2.out[0], hb3.in[0]);
		connect<>(hb3.out[0], chan.in[0]);
		connect<>(chan.out[0], out.in[0]);

	};
};

