//<copyright-disclaimer-start>
//  **************************************************************************************************************
//  * © 2025 Advanced Micro Devices, Inc. All rights reserved.                                                   *
//  * DISCLAIMER                                                                                                 *
//  * The information contained herein is for informational purposes only, and is subject to change              *
//  * without notice. While every precaution has been taken in the preparation of this document, it              *
//  * may contain technical inaccuracies, omissions and typographical errors, and AMD is under no                *
//  * obligation to update or otherwise correct this information.  Advanced Micro Devices, Inc. makes            *
//  * no representations or warranties with respect to the accuracy or completeness of the contents of           *
//  * this document, and assumes no liability of any kind, including the implied warranties of noninfringement,  *
//  * merchantability or fitness for particular purposes, with respect to the operation or use of AMD            *
//  * hardware, software or other products described herein.  No license, including implied or                   *
//  * arising by estoppel, to any intellectual property rights is granted by this document.  Terms and           *
//  * limitations applicable to the purchase or use of AMD’s products are as set forth in a signed agreement     *
//  * between the parties or in AMD's Standard Terms and Conditions of Sale. GD-18                               *
//  *                                                                                                            *
//  **************************************************************************************************************
//<copyright-disclaimer-end>


#include <adf.h>

#include "fir24_sym_param.h"

using namespace adf;

class adaptive_graph : public graph
{
public:
    port<direction::in> coefficients;
    input_plio in;

    // This is the hierachical port used to send output samples to the platform
    output_plio dataout;

    // Declare the filter kernel
    kernel fir24;

    adaptive_graph()
    {
		// Initialize the kernels
		fir24 = kernel::create(fir24_sym);

		in=input_plio::create("Datain0", plio_32_bits,  "data/input.txt");
		dataout=output_plio::create("Dataout0", plio_32_bits,  "data/output.txt");

		// Set a runtime ratio for the filter
		runtime<adf::ratio>(fir24) = 0.5;

		// Declare the sources for the two kernels
		source(fir24) = "kernels/hb24.cc";

		//TODO: connect filter coefficients
		connect< parameter > net0(coefficients, async(fir24.in[1]));

		connect< > net1(in.out[0], fir24.in[0]);
		connect< > net2(fir24.out[0], dataout.in[0]);
    }
};

