//<copyright-disclaimer-start>
//  **************************************************************************************************************
//  * © 2025 Advanced Micro Devices, Inc. All rights reserved.                                                   *
//  * DISCLAIMER                                                                                                 *
//  * The information contained herein is for informational purposes only, and is subject to change              *
//  * without notice. While every precaution has been taken in the preparation of this document, it              *
//  * may contain technical inaccuracies, omissions and typographical errors, and AMD is under no                *
//  * obligation to update or otherwise correct this information.  Advanced Micro Devices, Inc. makes            *
//  * no representations or warranties with respect to the accuracy or completeness of the contents of           *
//  * this document, and assumes no liability of any kind, including the implied warranties of noninfringement,  *
//  * merchantability or fitness for particular purposes, with respect to the operation or use of AMD            *
//  * hardware, software or other products described herein.  No license, including implied or                   *
//  * arising by estoppel, to any intellectual property rights is granted by this document.  Terms and           *
//  * limitations applicable to the purchase or use of AMD’s products are as set forth in a signed agreement     *
//  * between the parties or in AMD's Standard Terms and Conditions of Sale. GD-18                               *
//  *                                                                                                            *
//  **************************************************************************************************************
//<copyright-disclaimer-end>


#ifndef _GRAPH_H
#define _GRAPH_H

#include <adf.h>
#include <include.h>
#include "aie_kernels.h"

using namespace adf;

class clipped : public graph {

  private:
    kernel interpolator;
    kernel classify;

  public:

    adf::input_plio in;
    adf::output_plio out;

    adf::output_plio clip_in;
    adf::input_plio clip_out;

    clipped() {
      in  = adf::input_plio::create("DataIn1", plio_32_bits,"data/input.txt"); //Interpolator input
      out = adf::output_plio::create("DataOut1", plio_32_bits,"data/output.txt"); //Classifier output

      clip_out = adf::input_plio::create("polar_clip_in",plio_32_bits,"data/input1.txt"); //Classifier input
      clip_in = adf::output_plio::create("polar_clip_out", plio_32_bits,"data/output1.txt"); //Interpolator output

      interpolator = kernel::create(fir_27t_sym_hb_2i);
      classify     = kernel::create(classifier);

      connect<>(in.out[0], interpolator.in[0]);
      connect<>(interpolator.out[0], clip_in.in[0]);

      connect<>(clip_out.out[0], classify.in[0]);
      connect<>(classify.out[0], out.in[0]);

      dimensions(interpolator.in[0]) = {INTERPOLATOR27_INPUT_SAMPLES};
      dimensions(interpolator.out[0]) = {INTERPOLATOR27_OUTPUT_SAMPLES};
      dimensions(classify.out[0]) = {CLASSIFIER_OUTPUT_SAMPLES};
      
      std::vector<std::string> myheaders;
      myheaders.push_back("include.h");

      adf::headers(interpolator) = myheaders;
      adf::headers(classify) = myheaders;

      adf::source(interpolator) = "kernels/interpolators/hb27_2i.cc";
      adf::source(classify)    = "kernels/classifiers/classify.cc";

      runtime<ratio>(interpolator) = 0.4;
      runtime<ratio>(classify) = 0.4;
    };
};

#endif /* __GRAPH_H__ */

