//<copyright-disclaimer-start>
//  **************************************************************************************************************
//  * © 2025 Advanced Micro Devices, Inc. All rights reserved.                                                   *
//  * DISCLAIMER                                                                                                 *
//  * The information contained herein is for informational purposes only, and is subject to change              *
//  * without notice. While every precaution has been taken in the preparation of this document, it              *
//  * may contain technical inaccuracies, omissions and typographical errors, and AMD is under no                *
//  * obligation to update or otherwise correct this information.  Advanced Micro Devices, Inc. makes            *
//  * no representations or warranties with respect to the accuracy or completeness of the contents of           *
//  * this document, and assumes no liability of any kind, including the implied warranties of noninfringement,  *
//  * merchantability or fitness for particular purposes, with respect to the operation or use of AMD            *
//  * hardware, software or other products described herein.  No license, including implied or                   *
//  * arising by estoppel, to any intellectual property rights is granted by this document.  Terms and           *
//  * limitations applicable to the purchase or use of AMD’s products are as set forth in a signed agreement     *
//  * between the parties or in AMD's Standard Terms and Conditions of Sale. GD-18                               *
//  *                                                                                                            *
//  **************************************************************************************************************
//<copyright-disclaimer-end>


#include <ap_int.h>
#include <hls_stream.h>
#include <ap_axi_sdata.h>
#include <hls_math.h>

typedef ap_axis<32,0,0,0>  mycint16; 
short real(mycint16 &v) {
  return (short) (v.data);
} ;
short imag(mycint16 &v) {
  return (short) (v.data>>16);
} 

#define CFR_THRESHOLD (11626)
#define SIGN(x) (x>0 ? 0.5 : -0.5)


extern "C"
void polar_clip(hls::stream<mycint16>& input, hls::stream<mycint16>& output, int size)
{
    #pragma HLS interface axis port=input
    #pragma HLS interface axis port=output
    #pragma HLS interface s_axilite port=return bundle=control
    #pragma HLS interface ap_ctrl_hs port=return
	#pragma HLS interface s_axilite port=size bundle=control
    #pragma HLS interface ap_none port=size

	for (int i=0; i<size; i++)
	{
		mycint16 value = input.read();
		short rv = real(value);
		short im = imag(value);
		double ph = atan2((double)im,(double)rv) ;
		int mag_sq = rv*rv + im*im ;
		double mag = sqrt(double(mag_sq)) ;
		if (mag_sq>CFR_THRESHOLD*CFR_THRESHOLD)
		{
			double re = (mag-CFR_THRESHOLD)*cos(ph) ;
			double im = (mag-CFR_THRESHOLD)*sin(ph) ;     
			mycint16 value; 
			value.data = ((short)(im+SIGN(im)) << 16) | ((short)(re+SIGN(re)));
			output.write(value); 
		}
		else
		{
			mycint16 value;
			value.data=0; 
			output.write(value); 
		}
	}
}
